
# -----------------------------------------------------------------------------
# GiDCustomHelp --
#
# Html based help system. Show the help window.
#
# Arguments -- pairs of <option value>, where option could be:
#
#   -title : window title, defaults to "Help on <ProblemType name>"
#
#   -dir : path to help content. If -dir is missing it defaults to
#          "<ProblemType dir>/html". Multilingual content could be present, in
#          such case it is assumed that there is a directory for each language
#          provided. If the currente language is not found, language 'en'
#          (for english) is tried. Finally if 'en' is not found the value
#          provided to the option -dir is assumed as the base directory for the
#          help content.
#
#   -start : is a path relative to the value of -dir to an html link. For
#            instance:
#             -start html-version
#             -start html-tutorials/tutorial_1
#
#   -report : boolean value indicating if the the window format is report. If
#             report is requested no tree is showed, only the content pane is
#             showed.
#
# Structure of the help contents--
#

#   Assuming we have chosen html as the base directory for the multilingual
#   help contents we can have the following structure:
#
#      html
#       \__ en - English content
#
#       \__ es - Spanish content
#
#   Each content will provably have a directory structre to organize the
#   information. By default the help system build a tree resembling the
#   directory structre of the help content. In this sense there will be an
#   internal node for each subdirectory, finally the html documents will be
#   the terminal nodes of the tree.
#
#   We can also provide a help.conf configuration file in order to provide
#   more information about the estructure of the help. In a help file we can
#   specify a table of contents (TocPage), help subdirectories (HelpDirs)
#   and topic index (IndexPage).
#
# HelpDirs --
#
#  With HelpDirs we can specify wich of the subdirectories will be internal
#  nodes of the help tree. Also we can specify label for the node and a link
#  to load when the noded is clicked. The link is relative the node. For
#  instance:
#
#      HelpDirs {html-version "GiD Help" "intro/intro.html"} \
#               {html-customization "GiD Customization"} \
#               {html-faq "Frequently Asked Questions"} \
#               {html-tutorials "GiD Tutorials" "tutorials_toc.html"} \
#               {html_whatsnew "Whats New"}
#
# TocPage --
#
#  TocPage define an html page as a page describing a table of contents of
#  the current node (current directory). We have considered two ways for
#  specifying a table of content:
#
#     1- <UL> <LI> ... </UL> (default)
#     2- <DT> <DL> ... </DT>
#
#  The first is the one generated by texinfo.
#
#  For instance:
#
#    TocPage gid_toc.html
#
#    TolcPage contents.ht DT
#
# IndexPage --
#
#  If we specify by IndexPage a topic index we can take advantage of the
#  search index. In indexPage we can provide a set of html index pages along
#  with the structure type of the index. The type of the index could be:
#
#    1- <DIR> <LI> ... </DIR> (default)
#    2- <UL> <LI> ... </UL> (only one level of <UL>)
#
#  The first is the one generated by texinfo.
#
#  For instance:
#
#    IndexPage html-version/gid_18.html html-faq/faq_11.html
#
#
# help.conf example, taken from info dir in gid instalation:
#
#    HelpDirs \
#      {html-version "GiD Help" "intro/intro.html"} \
#      {html-customization "GiD Customization"} \
#      {html-faq "Frequently Asked Questions"} \
#      {html-tutorials "GiD Tutorials" "tutorials_toc.html"} \
#      {html_whatsnew "Whats New"}
#
#      IndexPage html-version/gid_18.html html-faq/faq_11.html
# -----------------------------------------------------------------------------

proc GiDCustomHelp { args } {
  array set options {
    -start ""
    -report 0
  }
  set lan [GidUtils::GetCurrentLanguage]
  set options(-dir) [file join [GetDirectoryProblemType] html]
  if { [file exists $options(-dir)] && [file isdirectory $options(-dir)] } {      
      set dirhelp [file join $options(-dir) $lan]
      if { ![file exists $dirhelp] && $lan != "en" } {
          set lan en
          set dirhelp [file join [GetDirectoryProblemType] html $lan]
      }
      if { [file exists $dirhelp] && [file isdirectory $dirhelp] } {
          set options(-dir) $dirhelp
      }
  }
  set options(-title) [_ "Help on %s" [GiD_Info Project Problemtype]]
  array set options $args
  GiDHelpWindow -type CUSTOM -dir $options(-dir) -start $options(-start) -report $options(-report) -title $options(-title)
}

proc GiDGetHelpLangDir { base_dir } {
  set lan [GidUtils::GetCurrentLanguage]
  set found 0
  foreach dl [list $lan en] {
    set lan_dir [file join $base_dir $dl]
    if {[file exists $lan_dir]} {
      set found 1
      break
    }
  }
  if {$found} {
    return $lan_dir
  }
  return $base_dir
}

proc GiDGetInfoDir { } {
  GiDGetHelpLangDir [file join $::GIDDEFAULT info]
}

proc ::pepe { } {
    WarnWinText lognoter--pepe
}

proc GiDOpenLognoter { args } {            
    package require compass_utils ;#for parse_args only
    set optional {
        { -open_page page_name "" }
        { -select_in_tree name "" }
        { -key key "" }
    }
    set compulsory dbfile    
    parse_args $optional $compulsory $args
    
    set execute_commands ""
    if { $open_page ne "" } {
        append execute_commands ";[list showpage $open_page]"
    }
    if { $select_in_tree ne "" } {
        append execute_commands ";[list toctree_process expand_page $select_in_tree]"
        append execute_commands ";[list toctree_process select_in_tree [list $select_in_tree]]"
    }   

    set lan [GidUtils::GetCurrentLanguage]
    
    set argv [list -readprefs 0 -readonly 1 -language $lan \
            -manageicontray 0 -prefs_data \
            [list view_languages_toolbar 0 add_more_pages_to_tree 0 \
                view_main_menu 0 geometry_default {{} 940x700 {tree 200}}] \
            -hide_on_exit 0 -execute_commands $execute_commands \
            -title [_ "Help viewer"] -saveprefs 0 \
            -dbtype sqlite -notebookname $dbfile -key $key]
    
    if { ![interp exists lognoter_intp] } {
        interp create lognoter_intp
        lognoter_intp eval lappend auto_path $::auto_path
        #lognoter_intp eval [list set auto_path $::auto_path]
        lognoter_intp eval [list set argv [list -language $lan]]
    }        

    package require Lognoter
   
    #interp alias lognoter_intp exit "" return
    #lappend argv -external_master_callback eval_master
    
    # necessary to avoid loading different versions in main interp and slave interp
    #lognoter_intp eval [list package require -exact treectrl [package require treectrl]]
    
    set r [lognoter_intp eval lognoter .%AUTO% $argv]    
    #interp alias lognoter_intp eval_master "" gid_groups_conds::_report_data_from_lognoter_cmd
    if { ![winfo exists .gid]  } {
	#not exit if opened from GiD
	interp alias lognoter_intp exit {} exit ;#else not exit or exit raise an error when closing lognoter       
    } else {
	if { $::tcl_platform(os) == "Darwin"} {
	    # if GiD exists, if lognoter exists, do not exit from GiD
	    proc ::DoNothing  {} {}
	    interp alias lognoter_intp exit {} ::DoNothing
	}
    }
}

proc GiDHelpWindowLognoter { {filename gid_reference_manual.wnl} } {
    if { [file pathtype $filename] == "relative" } {
        set full_filename [file join [GiDGetInfoDir] $filename]        
    } else {
        set full_filename $filename
    }
    if { [file exists $full_filename] } {        
        set open_page "" 
        set select_in_tree ""
        #package require gid_groups_conds
        #gid_groups_conds::open_help_viewer -open_page $open_page -select_in_tree $select_in_tree $full_filename   
        GiDOpenLognoter -open_page $open_page -select_in_tree $select_in_tree $full_filename
    }
}

proc GiDHelpWindow { args } {
  array set options {
    -type "REFERENCE"
    -dir ""
    -start ""
    -report 0
    -toplevel ""
    -tab tree
  }
  set options(-title) [_ "Help"]
  array set options $args

  switch [string toupper $options(-type)] {
    REFERENCE {
      set options(-dir) [GiDGetInfoDir]
      set options(-title) [_ "%s Reference" $::GidPriv(ProgName)]
    }
    REPORT {
      set options(-title) [_ "Report"]
      set options(-report) 1
    }
    CUSTOM {
      set options(-toplevel) .ctmhelp
    }
  }

  if {$options(-dir) eq ""} {
    error [_ "Calling GiDHelpWindow: should provide a -dir option"]
  }

  set ghv GiDHelpViewer::Show
  if { $::tcl_platform(os) == "Darwin"} {
      set ghv GiDHelpViewer3::Show
  }

  if {$options(-toplevel) eq ""} {
    $ghv $options(-dir) \
        -title $options(-title) \
        -try $options(-start) \
        -report $options(-report) \
        -tab $options(-tab)

  } else {
    $ghv $options(-dir) \
        -title $options(-title) \
        -try $options(-start) \
        -report $options(-report) \
        -base $options(-toplevel) \
        -tab $options(-tab)
  }
}

proc HelpWindow { help_type dir { intro_file "" } } {
    global var estat arbre finestra LastFile GIDDEFAULT

    GIDHELP_ini_var
    GIDHELP_setup $help_type


    set whlp $var(win)
    set top_whlp [winfo toplevel $whlp]
    set finestra $whlp.dreta.fondo.html



    if {$help_type == "REFERENCE"} {
        ini_ref
        wm geometry $top_whlp 800x550
        set var(path_manual) [file join [GiDGetInfoDir] html-version]
        set var(toc_file) gid_toc.html
        ShowContents
        if {$dir!=""} {generic_click $dir $whlp.dreta.fondo.html}
    }

    if {$help_type == "SEARCH"} {
        ini_ref
        
        set var(path_manual) [file join [GiDGetInfoDir] html-version]
        set var(toc_file) gid_toc.html
        ShowSearch
    }

    if {$help_type == "CUSTOMIZATION"} {
        ini_cust
        
        set LastFile [file join $var(path_manual) ptype_toc.html]
        IndexLink [file join $var(path_manual) ptype_toc.html] $whlp.dreta.fondo.html
        
    }

    if {$help_type == "FAQ"} {
        ini_faq
        
        set LastFile [file join $var(path_manual) faq_toc.html]
        IndexLink [file join $var(path_manual) faq_toc.html] $whlp.dreta.fondo.html
    }

    if {$help_type == "CUSTOM_HELP"} {
        ini_other $dir
        
        set LastFile [file join $var(path_manual) index.html]
        IndexLink [file join $var(path_manual) index.html] $whlp.dreta.fondo.html
    }

    if {$help_type == "CUSTOM_HELP_FILE" || $help_type == "TUTORIAL"} {
        
        ini_other_file $dir
        if {$help_type == "CUSTOM_HELP_FILE"} {wm geometry $top_whlp 679x580}
        if {$help_type == "TUTORIAL"} {wm geometry $top_whlp 600x500}
        
        set LastFile [file join $GIDDEFAULT $dir]
        IndexLink [file join $GIDDEFAULT $dir] $whlp.dreta.fondo.html
    }

    if {$help_type == "CUSTOM_HELP_REFERENCE"} {
        ini_ref
        
        set var(path_manual) [file dirname $dir]
        set var(toc_file) [file tail $dir]
        set var(search_prefix) "_Toc"
        
        set var(path_search_file) [file join $var(path_manual) search_data.txt]
        set var(path_cindex) ""
        if { $intro_file == "" } {
            set var(path_intro) $dir
        } else {
            set var(path_intro) $intro_file
        }
        
        ShowContents
        if {$dir!="" && $intro_file=="" } {
            generic_click $dir $whlp.dreta.fondo.html
        }
    }

}

proc directHelp {link} {
    global var estat
    #HelpWindow "REFERENCE" $link
    GiDHelpWindow -type REFERENCE -start [file join html-version $link]
}

proc ToolbarHelp {toolbar} {
    if {$toolbar == [_ "New %s project" $::GidPriv(ProgName)]} {
        return [list {gid13.html#FILES>New}]
    } elseif {$toolbar == [_ "Open %s project" $::GidPriv(ProgName)]} {
        return [list {gid14.html#FILES>Open}]
    } elseif {$toolbar == [_ "Save %s project" $::GidPriv(ProgName)]} {
        return  [list {gid15.html#FILES>Save}]
    } elseif {$toolbar == [_ "Previous view"] \
            || $toolbar == [_ "Next view"] \
            || $toolbar == [_ "List of views"]} {
        return  [list {gid26.html#VIEW>Zoom}]
    } elseif {$toolbar == [_ "Take a snapshot"]} {
        return [list {gid20.html#FILES>Print to file}]
    } elseif {$toolbar == [_ "Print current view"]} {
        return [list {gid22.html#FILES>Print}]
    } elseif {$toolbar == [_ "Toggle between pre and postprocess"]} {
        return [list {gid19.html#FILES>Preprocess/Postprocess}]
    } elseif {$toolbar == [_ "Copy/Transform"]} {
        return [list {gid54.html#UTILITIES>Copy}]
    } elseif {$toolbar == [_ "Layers"]} {
        return [list {gid52.html#UTILITIES>Layers}]
    } elseif {$toolbar == [_ "Preferences"]} {
        return [list {gid51.html#UTILITIES>Preferences}]
    } elseif {$toolbar == [_ "Help"]} {
        return [list {gid99.html#HELP}]
    } elseif {$toolbar == [_ "Exit %s" $::GidPriv(ProgName)]} {
        return [list {gid24.html#FILES>Quit}]
    } elseif {$toolbar == [_ "Zoom in"] \
            || $toolbar == [_ "Zoom out"] \
            || $toolbar == [_ "Zoom frame"]} {
        return [list {gid26.html#VIEW>Zoom}]
    } elseif {$toolbar == [_ "Redraw"]} {
        return [list {gid29.html#VIEW>Redraw}]
    } elseif {$toolbar == [_ "Rotate trackball"]} {
        return [list {gid27.html#VIEW>Rotate>Rotate trackbal}]
    } elseif {$toolbar == [_ "Pan dynamic"]} {
        return [list {gid28.html#VIEW>Pan}]
    } elseif {$toolbar == [_ "Create line"]} {
        return [list  {gid46.html#GEOMETRY>Create>Straight line creation}]
    } elseif {$toolbar == [_ "Create arc"]} {
        return [list {gid46.html#GEOMETRY>Create>Arc creation}]
    } elseif {$toolbar == [_ "Create nurbs line"]} {
        return [list {gid46.html#GEOMETRY>Create>NURBS line creation}]
    } elseif {$toolbar == [_ "Create polyline"]} {
        return [list {gid46.html#GEOMETRY>Create>Polyline creation}]
    } elseif {$toolbar == [_ "Create NURBS surface"]} {
        return [list {gid46.html#GEOMETRY>Create>NURBS surface creation}]
    } elseif {$toolbar == [_ "Create volume"]} {
        return [list {gid46.html#GEOMETRY>Create>Volume creation}]
    } elseif {$toolbar == [_ "Create Object:\n  Rectangle\n Polygon\n  Circle\n  Sphere\n  Cylinder\n  Cone\n  Prism\n Thorus"]} {
        return [list {gid46.html#GEOMETRY>Create>Object}]
    } elseif {$toolbar == [_ "Delete"]} {
        return [list {gid47.html#GEOMETRY>Delete}]
    } elseif {$toolbar == [_ "List entities"] } {       
        if { [GetCurrentPrePostMode] == "PRE" } {
            return [list {gid57.html#UTILITIES>List}]
        } else {
            return [list {gid113.html#POSTPROCESS OPTIONS>Utilities menu}]
        }
    } elseif {$toolbar == [_ "Toggle geometry-mesh view"]} {
        return [list {gid43.html#VIEW>Mode}]
    } elseif {$toolbar == [_ "Change light vector direction"] } {
        return [list {gid30.html#VIEW>Render}]
    } elseif {$toolbar == [_ "Display Style"] \
            || $toolbar == [_ "View style"] \
            || $toolbar == [_ "Culling Style"] \
            || $toolbar == [_ "Switch Volume sets On/Off"] \
            || $toolbar == [_ "Switch Surface set On/Off"] \
            || $toolbar == [_ "Switch Cut set On/Off"]} {
        return [list {gid116.html#POSTPROCESS OPTIONS>Display Style}]
    } elseif {$toolbar == [_ "Cut Volume/Surface sets with 2/3 points, succesion#C#menu"]} {
        return [list {gid114.html#POSTPROCESS RESULTS>Do Cuts}]
    } elseif {$toolbar == [_ "Set maximum value"] \
            || $toolbar == [_ "Set minimum value"] \
            || $toolbar == [_ "Reset Contour Limit Values"]} {
        return [list {gid124.html#POSTPROCESS RESULTS>Contour Fill}]
    } elseif {$toolbar == [_ "Record macro"] \
            || $toolbar == [_ "Edit macros..."] } {
        return [list {gid53.html#UTILITIES>Tools>Macros}]
    } elseif {$toolbar == [_ "Open a simple editor to save small notes with the model"]} {
        return [list {gid53.html#UTILITIES>Tools>Notes}]
    } elseif {$toolbar == [_ "Dimension distance between two points"] \
            || $toolbar == [_ "Modify the dimension text"]} {
        return [list {gid63.html#UTILITIES>Dimensions}]
    } elseif {$toolbar == [_ "Create a trimmed surface from a base surface and a new boundary"]} {
        return [list {gid46.html#GEOMETRY>Create>NURBS surface creation}]
    } elseif {$toolbar == [_ "Create a hole in a surface from a line's loop "]} {
        return [list {gid48.html#GEOMETRY>Edit>Hole NURBS surface}]
    } elseif {$toolbar == [_ "Move a point (Warning: surfaces can't support big deformation)"]} {
        return [list {gid48.html#GEOMETRY>Edit>Move point}]
    } elseif {$toolbar == [_ "Divide a line in a number of divisions"]} {
        return [list {gid48.html#GEOMETRY>Edit>Divide}]
    }
    return {}
}

# tutorials ini window#
#NOT IN USE
#proc CreateTutorial_win {} {
#    global GIDDEFAULT GidPriv
#
#    set w .gid.tutorials
#    InitWindow $w [_ "%s tutorials" $::GidPriv(ProgName)] GiDTutorialsWindowGeom CreateTutorial_win "" 1
#    if { ![winfo exists $w] } return ;# windows disabled || usemorewindows == 0
#    $w configure -background #EEEEEE
#    #wm minsize $w 430 430
#    #wm maxsize $w 430 430
#    set dir [file join [GiDGetInfoDir] html-tutorials images_1]
#    set image_tutorials [image create photo -file [file join $dir gidtutorials.gif]]
#    set image_tutorial1 [image create photo -file [file join $dir tutorial1.gif]]
#    set image_tutorial2 [image create photo -file [file join $dir tutorial2.gif]]
#    set image_tutorial3 [image create photo -file [file join $dir tutorial3.gif]]
#
#    set image_tutorial1_over [image create photo -file [file join $dir tutorial1_o.gif]]
#    set image_tutorial2_over [image create photo -file [file join $dir tutorial2_o.gif]]
#    set image_tutorial3_over [image create photo -file [file join $dir tutorial3_o.gif]]
#
#    ttk::label $w.label_imgtop -image $image_tutorials -borderwidth 0
#
#    tk::button $w.button#1 \
#            -relief flat \
#            -image $image_tutorial1 -background #EEEEEE \
#            -command {
#        GiDHelpWindow -type REFERENCE -start [file join html-tutorials "tutorials01.html#Preprocess Tutorial 1"]
#        destroy .gid.tutorials
#    }
#
#
#    tk::button $w.button#2 \
#            -relief flat \
#            -image $image_tutorial2 -background #EEEEEE \
#            -command {
#        GiDHelpWindow -type REFERENCE  -start [file join html-tutorials "tutorials09.html#Preprocess Tutorial 2"]
#        destroy .gid.tutorials
#    }
#
#    tk::button $w.button#3 \
#            -relief flat \
#            -image $image_tutorial3 -background #EEEEEE \
#            -command {
#        GiDHelpWindow -type REFERENCE -start [file join html-tutorials "tutorials16.html#Meshing Tutorial 1"]
#        destroy .gid.tutorials
#    }
#
#    bind $w.button#1 <Enter> ".gid.tutorials.button#1 configure -image $image_tutorial1_over"
#    bind $w.button#1 <Leave> ".gid.tutorials.button#1 configure -image $image_tutorial1"
#
#    bind $w.button#2 <Enter> ".gid.tutorials.button#2 configure -image $image_tutorial2_over"
#    bind $w.button#2 <Leave> ".gid.tutorials.button#2 configure -image $image_tutorial2"
#
#    bind $w.button#3 <Enter> ".gid.tutorials.button#3 configure -image $image_tutorial3_over"
#    bind $w.button#3 <Leave> ".gid.tutorials.button#3 configure -image $image_tutorial3"
#
#
#    # Geometry management
#
#    grid $w.label_imgtop -in $w        -row 0 -column 0 -columnspan 2 -sticky ne
#    grid $w.button#1 -in $w        -row 1 -column 0
#    grid $w.button#2 -in $w        -row 1 -column 1
#    grid $w.button#3 -in $w        -row 2 -column 0 -columnspan 2
#
#
#    # Resize behavior management
#
#    grid rowconfigure $w 1 -weight 1 -minsize 30
#    #grid rowconfigure $w 2 -weight 0 -minsize 174
#    #grid columnconfigure $w 1 -weight 0 -minsize 199
#    #grid columnconfigure $w 2 -weight 0 -minsize 253
#
#    #grid rowconfigure $w.frame 1 -weight 0 -minsize 30
#    #grid columnconfigure $w.frame 1 -weight 0 -minsize 30
#
#    if { $::tcl_platform(os) != "Darwin"} {
#        tkwait visibility $w
#    } else {
#        update
#    }
#
#    grab $w
#}

proc CreateGiDInfo_win {} {
    global GIDDEFAULT GidPriv tcl_platform

    set w .gid.info_gid
    InitWindow $w [_ "%s Info" $::GidPriv(ProgName)] GiDInfoWindowGeom CreateGiDInfo_win "" 1
    if { ![winfo exists $w] } return ;# windows disabled || usemorewindows == 0
    
    set colorbackground #ced1d6
    
    $w configure -background $colorbackground
    #wm minsize $w 459 330
    #wm maxsize $w 459 330

    set image_top [gid_themes::GetImage gidinfo.png] ;#now not include any version logo

    set rootname [file rootname $::GidPriv(LogosGiDBitmapName)]
    set extension [file extension $::GidPriv(LogosGiDBitmapName)]
    set size 64
    set src [gid_themes::GetImage ${rootname}${size}${extension}]
    set bd 10
    set x [expr [image width $image_top]-[image width $src]-$bd]
    $image_top copy $src -to $x $bd

    set image_but1 [gid_themes::GetImage buton_about.png]
    set image_but2 [gid_themes::GetImage buton_help.png]
    set image_but3 [gid_themes::GetImage buton_tutorials.png]
    set image_but4 [gid_themes::GetImage buton_goto.png]
    set image_but5 [gid_themes::GetImage buton_register.png]
    set image_but6 [gid_themes::GetImage buton_atsign.png]

    set image_but1_over [gid_themes::GetImage buton_about_o.png]
    set image_but2_over [gid_themes::GetImage buton_help_o.png]
    set image_but3_over [gid_themes::GetImage buton_tutorials_o.png]
    set image_but4_over [gid_themes::GetImage buton_goto_o.png]
    set image_but5_over [gid_themes::GetImage buton_register_o.png]
    set image_but6_over [gid_themes::GetImage buton_atsign_o.png]


    set image_txt1 [gid_themes::GetImage text_about.png]
    set image_txt2 [gid_themes::GetImage text_gidhelp.png]
    set image_txt3 [gid_themes::GetImage text_gidtutorials.png]
    set image_txt4 [gid_themes::GetImage text_www.png]
    set image_txt5 [gid_themes::GetImage text_register.png]
    set image_txt6 [gid_themes::GetImage text_contact.png]
    #set image_txt7 [gid_themes::GetImage text_customization.png]
    #set image_txt8 [gid_themes::GetImage text_whatsnew.png]
    #set image_txt9 [gid_themes::GetImage text_faq.png]

    tk::label $w.label_imgtop -image $image_top -borderwidth 0
    tk::label $w.label_about -image $image_txt1 -borderwidth 0  -background $colorbackground
    tk::label $w.label_help -image $image_txt2 -borderwidth 0  -background $colorbackground
    tk::label $w.label_tutorials -image $image_txt3 -borderwidth 0  -background $colorbackground
    tk::label $w.label_web -image $image_txt4 -borderwidth 0  -background $colorbackground
    tk::label $w.label_register -image $image_txt5 -borderwidth 0  -background $colorbackground
    tk::label $w.label_close -image $image_txt6 -borderwidth 0  -background $colorbackground

    tk::button $w.button#1 -image $image_but1 -command {CreateWelcomeWin } -borderwidth 0
    tk::menubutton $w.button#2 -image $image_but2 -borderwidth 0 -menu $w.button#2.m 
    menu $w.button#2.m -tearoff no -borderwidth 1 
    $w.button#2.m add command -label [_ "%s Help" $::GidPriv(ProgName)] -command {GiDHelpWindowLognoter gid_reference_manual.wnl}
    $w.button#2.m add command  -label [_ "Customization Help"] -command {GiDHelpWindowLognoter gid_customization_manual.wnl}
    $w.button#2.m add command -label [_ "FAQ"] -command {GiDHelpWindowLognoter gid_FAQ.wnl}
    $w.button#2.m add command  -label [_ "What's new"]...  -command {GiDHelpWindowLognoter gid_what_s_new.wnl}

    tk::button $w.button#3 -image $image_but3 -borderwidth 0 -command {         
        GiDHelpWindowLognoter gid_user_manual.wnl 
        destroy .gid.info_gid
    }
    tk::button $w.button#4 -image $image_but4 -command [list VisitWeb $GidPriv(ProgWeb)] -borderwidth 0


    tk::menubutton $w.button#5 -image $image_but5 -menu $w.button#5.m -borderwidth 0

    menu $w.button#5.m -tearoff no -borderwidth 1
    $w.button#5.m add command -label [_ "Register %s" $::GidPriv(ProgName)] \
            -command {gidregister}
    $w.button#5.m add command  -label [_ "Register Problem type"] \
            -command {gidregister module}


    tk::button $w.button#6 -image $image_but6 -command [list VisitWeb mailto:$GidPriv(ContactMail)] -borderwidth 0

    #binds - on mouse over

    bind $w.button#1 <Enter> ".gid.info_gid.button#1 configure -image $image_but1_over"
    bind $w.button#1 <Leave> ".gid.info_gid.button#1 configure -image $image_but1 "

    bind $w.button#2 <Enter> ".gid.info_gid.button#2 configure -image $image_but2_over"
    bind $w.button#2 <Leave> ".gid.info_gid.button#2 configure -image $image_but2 "

    bind $w.button#3 <Enter> ".gid.info_gid.button#3 configure -image $image_but3_over"
    bind $w.button#3 <Leave> ".gid.info_gid.button#3 configure -image $image_but3 "

    bind $w.button#4 <Enter> ".gid.info_gid.button#4 configure -image $image_but4_over"
    bind $w.button#4 <Leave> ".gid.info_gid.button#4 configure -image $image_but4 "

    bind $w.button#5 <Enter> ".gid.info_gid.button#5 configure -image $image_but5_over"
    bind $w.button#5 <Leave> ".gid.info_gid.button#5 configure -image $image_but5 "

    bind $w.button#6 <Enter> ".gid.info_gid.button#6 configure -image $image_but6_over"
    bind $w.button#6 <Leave> ".gid.info_gid.button#6 configure -image $image_but6 "

    # Geometry management

    grid $w.label_imgtop -in $w -row 1 -column 1 -columnspan 6 -sticky ne
   
    grid $w.button#1        -in $w        -row 2 -column 2
    grid $w.label_about        -in $w        -row 2 -column 3
    grid $w.button#3        -in $w        -row 2 -column 4 -pady 5
    grid $w.label_tutorials  -in $w        -row 2 -column 5

    grid $w.button#2        -in $w        -row 3 -column 2 -pady 5
    grid $w.label_help        -in $w        -row 3 -column 3

    grid $w.button#5        -in $w        -row 3 -column 4 -pady 5
    grid $w.label_register        -in $w        -row 3 -column 5
    grid $w.button#4        -in $w        -row 4 -column 2 -pady 5
    grid $w.label_web        -in $w        -row 4 -column 3
    grid $w.button#6        -in $w        -row 4 -column 4 -pady 5
    grid $w.label_close        -in $w        -row 4 -column 5


    # Resize behavior management

    grid rowconfigure $w 1 -weight 1
    #grid rowconfigure $w 2 -weight 0 -minsize 2
    grid rowconfigure $w 5 -weight 0 -minsize 10

    grid columnconfigure $w 1 -weight 0 -minsize 15
    grid columnconfigure $w 2 -weight 0 -minsize 30
    grid columnconfigure $w 3 -weight 0 -minsize 30
    grid columnconfigure $w 4 -weight 0 -minsize 30
    grid columnconfigure $w 5 -weight 0 -minsize 40
    grid columnconfigure $w 6 -weight 0 -minsize 10
}


# whatisnewViwer is not used any more. see function HelpWindow "CUSTOM_HELP_FILE" #
proc WhatIsNewViewer { { name  "" } } {
    global GIDDEFAULT

    if { $name == "" } {
        set name $GIDDEFAULT/info/WHATISNEW
    }

    set w .gid.__fileviewer
    InitWindow $w [_ "What is new"] WhatIsNewWindowGeom WhatIsNewViewe
    if { ![winfo exists $w] } return ;# windows disabled || usemorewindows == 0

    $w configure -background grey93

    set imatge_quit [gid_themes::GetImage quit.png]




    text $w.t -relief raised -borderwidth 2 -xscrollcommand "$w.sx set" \
            -yscrollcommand "$w.sy set"
    ttk::scrollbar $w.sy -command "$w.t yview"
    ttk::scrollbar $w.sx -command "$w.t xview" -orient h
    ttk::button $w.b -text [_ "Close"] -command "destroy $w" -image [gid_themes::GetImage quit.png generic] -style BottomFrame.TButton
    #-background grey93 -relief flat
    #SetWidgetsWidthFromText w.b

    grid $w.t -row 1 -column 1 -sticky nsew
    grid $w.sy -row 1 -column 2 -sticky ns
    #grid $w.sx -row 2 -column 1 -sticky ew
    grid $w.b -row 3 -column 1 -columnspan 2 -pady 7 -sticky e

    grid columnconf $w 1 -weight 1
    grid rowconf $w 1 -weight 1

    set f [open $name r]
    while { ![eof $f] } {
        $w.t insert end [gets $f]\n
    }
    close $f
    focus $w.t
    $w.t configure -state disabled
}


proc VisitWeb { dir } {
    global tcl_platform

    if { $tcl_platform(platform) == "windows" } {
        catch {eval exec [auto_execok start] $dir >& NUL: &}
    } elseif { $::tcl_platform(os) == "Darwin"} {
        set err ""
        set hay_error 0
        set hay_error [ catch { exec open "$dir"} err]
        if { $hay_error && $err != ""} {
            WarnWin $err
        }
    } else {
        #catch {exec netscape $dir >& /dev/null &}
        set err_txt ""
        set hay_error 0
        set lst_browsers [ list firefox chromium-browser mozilla opera konqueror]
        foreach browser $lst_browsers {
            set hay_error [ catch {exec firefox $dir >& /dev/null &} err_txt]
            if { !$hay_error} {
                break
            }
        }
        if { $hay_error && ( $err_txt != "")} { 
            WarnWin "$err_txt"
        }
    }
}

